/* Copyright 2020 Emanuele Sorce
 *
 * This file is part of Sturm Reader and is distributed under the terms of
 * the GPL. See the file COPYING for full details.
 */

import QtQuick.Controls 2.5
import QtQuick 2.12
import QtQuick.Layouts 1.3

Page {
	header: ToolBar {
		id: aboutheader
		width: parent.width
		RowLayout {
			spacing: scaling.dp(10)
			anchors.fill: parent

			ToolbarIcon {
				name: "go-previous"
				onClicked: pageStack.pop()
			}

			Label {
				text: gettext.tr("About")
				font.pixelSize: headerTextSize
				elide: Label.ElideRight
				horizontalAlignment: Qt.AlignLeft
				verticalAlignment: Qt.AlignVCenter
				Layout.fillWidth: true
				Layout.fillHeight: true
			}
		}
	}

	Flickable {
		id: flickable
		anchors.fill: parent
		contentHeight:  layout.height + scaling.dp(80)
		contentWidth: parent.width
		ScrollBar.vertical: ScrollBar { }

		Column {
			id: layout

			spacing: scaling.dp(35)
			anchors.top: parent.top
			anchors.left: parent.left
			anchors.right: parent.right
			width: parent.width

			Item {
				height: scaling.dp(30)
			}

			Image {
				anchors.horizontalCenter: parent.horizontalCenter
				height: width
				width: Math.min(parent.width/2, parent.height/3)
				source: Qt.resolvedUrl("../sturmreader.svg")
			}

			Label {
				anchors.horizontalCenter: parent.horizontalCenter
				width: parent.width * 4 / 5
				font.pixelSize: scaling.dp(25)
				font.bold: true
				horizontalAlignment: Text.AlignHCenter
				wrapMode: Text.WordWrap
				text: gettext.tr("Sturm Reader")
			}

			Label {
				anchors.horizontalCenter: parent.horizontalCenter
				width: parent.width * 4 / 5
				wrapMode: Text.WordWrap
				horizontalAlignment: Text.AlignHCenter
				text: gettext.tr("Sturm (und Drang) Reader is an open source Ebook reader.<br>Community is what makes this app possible, so pull requests, translations, feedback and donations are very appreciated :)<br>This app Is a fork of the Beru app by Rshcroll, Thanks!<br>This app stands on the shoulder of various Open Source projects, see source code for licensing details");
			}

			Label {
				anchors.horizontalCenter: parent.horizontalCenter
				width: parent.width * 4 / 5
				wrapMode: Text.WordWrap
				horizontalAlignment: Text.AlignHCenter
				text: gettext.tr("A big thanks to all translators, beta-testers and users<br/>in general, who improve this app with their work and feedback")
			}

			Label {
				anchors.horizontalCenter: parent.horizontalCenter
				width: parent.width * 4 / 5
				wrapMode: Text.WordWrap
				horizontalAlignment: Text.AlignHCenter
				text: gettext.tr("A special thanks to Joan Cibersheep for the new logo")
			}

			Label {
				anchors.horizontalCenter: parent.horizontalCenter
				width: parent.width * 4 / 5
				wrapMode: Text.WordWrap
				horizontalAlignment: Text.AlignHCenter
				text: gettext.tr("A special thanks to Jeroen for support and a test device")
			}

			Button {
				anchors.horizontalCenter: parent.horizontalCenter
				text: gettext.tr("❤Donate❤")
				highlighted: true
				width: parent.width * 4 / 5
				onClicked: Qt.openUrlExternally("https://paypal.me/emanuele42");
			}

			Button {
				anchors.horizontalCenter: parent.horizontalCenter
				text: gettext.tr("See source on:") + " Gitlab"
				highlighted: false
				width: parent.width * 4 / 5
				onClicked: Qt.openUrlExternally("https://gitlab.com/tronfortytwo/sturmreader");
			}

			Button {
				anchors.horizontalCenter: parent.horizontalCenter
				text: gettext.tr("Report bug or feature request")
				highlighted: true
				width: parent.width * 4 / 5
				onClicked: Qt.openUrlExternally("https://gitlab.com/TronFortyTwo/sturmreader/-/issues");
			}

			Button {
				anchors.horizontalCenter: parent.horizontalCenter
				text: gettext.tr("See License (GNU GPL v3)")
				width: parent.width * 4 / 5
				onClicked: Qt.openUrlExternally("https://gitlab.com/TronFortyTwo/sturmreader/-/blob/master/LICENSE");
			}

			Label {
				anchors.horizontalCenter: parent.horizontalCenter
				width: parent.width * 4 / 5
				wrapMode: Text.WordWrap
				horizontalAlignment: Text.AlignHCenter
				text: "Copyright (C) 2018-2021 Emanuele Sorce (emanuele.sorce@hotmail.com)"

			}

			Label {
				anchors.horizontalCenter: parent.horizontalCenter
				width: parent.width * 4 / 5
				wrapMode: Text.WordWrap
				horizontalAlignment: Text.AlignHCenter
				text: "Copyright (C) 2015 Robert Schroll"
			}
		}
	}
}
